var os = require('os');
var cpu = require('windows-cpu');
var ArgumentParser = require('argparse').ArgumentParser;
var fs = require('fs');
var helpBy = require('helpBy');
var dgram = require('dgram');

var morelog = require('morelog');
var log = new morelog('systemMonitor-client', 'cyan');

const toGB = 1073741824;

var SystemMonitor = function(ip, port) {

    this.m_ip = ip;
    this.m_port = port;

    this.m_data = {};
    //get the hostname
    this.m_data.hostname = os.hostname();

    this.m_data.platform = this._getPlatform();
    //this.m_data.architecture = this._getArch();
    this.m_data.uptime = this.checkUptime();
    this.m_data.memory = this._getMemoryInfo();
    this.m_data.cpu = this._getCpuInfo();
    //this.nwif = this._getNetworkInformation();

    this._initSocket();
    var that = this;

    setInterval(function(){
        that.update();
    }, 1000);
};

SystemMonitor.prototype._initSocket = function() {

    if(this.m_socket)
        this.m_socket.close();

    var that = this;
    this.m_socket = dgram.createSocket('udp4');

    this.m_socket.on('error', function(err){
        log.error("Error on UDP Socket");
        setTimeout(function(){
            that._initSocket();
        }, 500);
    });
}

SystemMonitor.prototype._send = function() {

    var data = JSON.stringify(this.m_data);
    this.m_socket.send(data, 0, data.length,  this.m_port, this.m_ip);
}

SystemMonitor.prototype._getPlatform = function() {
    var platform = os.platform()
    switch (platform) {
        case 'win32':
            platform = 'Windows';
            break;
        case 'linux':
            platform = 'Linux';
            break;
        case 'sunos':
            platform = 'SunOS'
            break;
        default:
            platform = 'Unknow Operating System'
    }
    return platform
}

SystemMonitor.prototype._getArch = function() {
    var that = this;
    var arch = os.arch();
    if (arch == "ARM")
        return 0, "this is not a pi!"
}

SystemMonitor.prototype._getCpuInfo = function() {

    var os_cpu = os.cpus();
    var cpu = [];

    for(index in os_cpu){
        var concreteCpu = os_cpu[index];
        var current = {};
        current.name = "cpu"+index;
        current.speed = concreteCpu.speed;
        current.model = concreteCpu.model;
        current.load;
        current.initialLoad = concreteCpu.times;
        cpu.push(current);
    }
    return cpu;
};

SystemMonitor.prototype._getMemoryInfo = function() {
    var memory = {};

    memory.totalmem = (os.totalmem() / toGB).toFixed(1);
    memory.freemem;
    memory.usedmem;
    memory.percentage;

    return memory;
};

SystemMonitor.prototype._getNetworkInformation = function() {
    var os_nwif = os.networkInterfaces();
    var nwif = {};
    if (os_nwif['WLAN']) {
        nwif.type = 'WLAN'
        os_nwif = os_nwif['WLAN']
    }
    if (os_nwif['Ethernet']) {
        nwif.type = 'Ethernet'
        os_nwif = os_nwif['Ethernet']
    }

    return nwif
};

SystemMonitor.prototype.checkUptime = function() {

    var seconds = os.uptime();
    var hours = Math.floor(seconds / 3600);
    var minutes = Math.floor((seconds - hours * 3600) / 60);
    seconds = Math.floor(seconds - hours * 3600 - minutes * 60);
    this.m_data.uptime = helpBy.zerosInFront(hours, 2) +
        ":" +
        helpBy.zerosInFront(minutes, 2) +
        ":" +
        helpBy.zerosInFront(seconds, 2);
};

SystemMonitor.prototype.checkMemory = function() {
    var memory = this.m_data.memory;
    memory.freemem = (os.freemem() / toGB).toFixed(1);
    memory.usedmem = (memory.totalmem - memory.freemem).toFixed(1);
    memory.percentage = Math.round((100 / memory.totalmem) * memory.usedmem);
};

SystemMonitor.prototype.checkLoad = function() {
    var that = this;
    if (this.m_data.platform == 'Windows') {
        cpu.totalLoad(
            function(error, results) {
                if (error)
                    return console.log(error);
                that.data.cpu.load = results;
            });
    }
    else {
        this._getLoad();
    }
};

SystemMonitor.prototype._getLoad = function() {

    var os_cpu = os.cpus();
    var stat = [];

    for(cpu in os_cpu){
        var concreteCpu = os_cpu[cpu];
        var current = this.m_data.cpu[cpu];
        var data = concreteCpu.times;
        var total = (data.user - current.initialLoad.user)
            + (data.nice - current.initialLoad.nice)
            + (data.sys - current.initialLoad.sys)
            + (data.idle - current.initialLoad.idle)
        var percent = ((total - (data.idle - current.initialLoad.idle)) / total) * 100;
        current.load = percent.toFixed(2)
    }
}

SystemMonitor.prototype.update = function(callback) {
    this.checkLoad();
    this.checkUptime();
    this.checkMemory();
    this._send();
    // console.log(this.m_data.cpu.load)
    if (callback)
        callback(this);

};

function main(){
    var parser = new ArgumentParser({
        version: '0.0.1',
        addHelp: true,
        description: "Argument parser"
    });
    //adding all arguments

    parser.addArgument(
        ['-aip', '--asgardIP'],
        {
            help: 'IP Adress of the Asgard - Server',
            defaultValue: undefined,

        });

    parser.addArgument(
        ['-p', '--port'],
        {
            help: 'Receive port at the Asgard Server',
            defaultValue: undefined,

        });
    var args = parser.parseArgs();
    var ip = args.asgardIP;
    var port = args.port
    if(port && ip){
        new SystemMonitor(args.asgardIP, args.port);
    }
    else {
        log.error("IP and Port needed!");
    }
}

main();
